import os
import unittest
import json
import tests.unit.test_fixture as fixture
from datetime import datetime
from crawling_gocd.gocd_domain import Organization
from crawling_gocd.crawler import Crawler
from unittest.mock import MagicMock


class CrawlerTest(unittest.TestCase):
    def setUp(self):
        self.orgnization = Organization("test.com", "username", "password")
        self.crawler = Crawler(self.orgnization)
        self.filePage1 = "tests/unit/resources/pipeline_history_pg_1.json"
        self.filePage2 = "tests/unit/resources/pipeline_history_pg_2.json"

    def test_generate_pipeline_history_url_correctly(self):
        url = self.crawler.generatePipelineHistoryUrl("go_service", 10)
        self.assertEqual(
            "https://test.com/go/pipelineHistory.json?pipelineName=go_service&start=10", url)

        url = self.crawler.generatePipelineHistoryUrl("go_service", 0)
        self.assertEqual(
            "https://test.com/go/pipelineHistory.json?pipelineName=go_service", url)

    def test_generate_pipeline_history_url_failed(self):
        url = self.crawler.generatePipelineHistoryUrl("go_service", 10)
        self.assertNotEqual(
            "https://test.com/go/pipelineHistory.json?pipelineName=go_service", url)

    def test_should_return_true_when_data_is_over_time(self):
        pipelineHistories = fixture.getPipelineHistories(self.filePage1)
        self.assertTrue(self.crawler.canStop(pipelineHistories, datetime.fromtimestamp(1567075172220/1000)))

    def test_should_return_false_when_data_is_not_over_time(self):
        pipelineHistories = fixture.getPipelineHistories(self.filePage1)
        self.assertFalse(self.crawler.canStop(pipelineHistories, datetime.fromtimestamp(1567075172200/1000)))

    def test_should_filter_data_when_data_is_over_time(self):
        pipelineHistories = fixture.getPipelineHistories(self.filePage1)
        result=self.crawler.filterPipelinesPerPage(pipelineHistories, datetime.fromtimestamp(1567075172220 / 1000), datetime.fromtimestamp(1567335377730/1000))
        self.assertEqual(len(result), 8)

    def test_should_get_pipeline_history_correctly(self):
        def side_effect(arg):
            if arg == "https://test.com/go/pipelineHistory.json?pipelineName=go_service&start=10":
                jsonFile=self.filePage2
            else:
                jsonFile=self.filePage1
            with open(jsonFile, 'r') as f:
                return json.load(f)

        self.crawler.getResource=MagicMock(side_effect=side_effect)
        pipelineHistories=self.crawler.getPipelineHistories("go_service", datetime.fromtimestamp(1567052779277/1000), datetime.fromtimestamp(1567335377730/1000))
        self.assertEqual(len(pipelineHistories), 16)

if __name__ == '__main__':
    unittest.main()



import json
import itertools
import datetime
from crawling_gocd.crawler import CrawlingDataMapper
from crawling_gocd.gocd_domain import Pipeline
from crawling_gocd.calculate_domain import InputsCalcConfig
from crawling_gocd.calculate_domain import Result

def getPipelineHistories(filePath):
    with open(filePath, 'r') as f:
        page1 = json.load(f)
    pipelineHistoriesList = list(map(lambda x: x["history"], page1["groups"]))
    return list(itertools.chain(*pipelineHistoriesList))

def generatePipeline():
    filePage1 = "tests/unit/resources/pipeline_history_pg_1.json"
    pipelineHistories = getPipelineHistories(filePage1)
    mapper = CrawlingDataMapper()

    pipeline = Pipeline("go_service", InputsCalcConfig({"qa": ["flyway-qa", "deploy-qa"]}, datetime.datetime(1970, 1, 1), datetime.datetime.now()))
    pipeline.setHistories(mapper.mapPipelineHistory(pipelineHistories))
    return pipeline

def getResults():
    return [
        Result("account-management-normal-master", "DeploymentFrequency", "ci", "145"),
        Result("account-management-normal-master", "DeploymentFrequency", "qa", "61"),
        Result("account-management-normal-master", "ChangeFailPercentage", "ci", "5.5%"),
        Result("account-management-normal-master", "ChangeFailPercentage", "qa", "4.9%"),
        Result("account-management-normal-master", "MeanTimeToRestore", "ci", "56(mins)"),
        Result("account-management-normal-master", "MeanTimeToRestore", "qa", "53(mins)"),
    ]


import yaml
import sys
import logging
from datetime import datetime, timedelta
from crawling_gocd.gocd_domain import Pipeline
from crawling_gocd.calculate_domain import InputsCalcConfig, GlobalTimeRange


class InputsParser:
    def __init__(self, filePath):
        self.inputTimeParser = InputTimeParser()
        with open(filePath, 'r') as stream:
            try:
                self.inputs = yaml.safe_load(stream)
            except yaml.YAMLError as exc:
                print("yaml file read failed, {}", exc)

    def parsePipelineConfig(self):
        globalTimeRange = self.getGlobalTimeRange()
        logging.info("global_start_time: {}, global_end_time: {}".format(globalTimeRange.startTime, globalTimeRange.endTime))

        return list(map(lambda pipeline: self.mapSinglePipeline(
            pipeline, globalTimeRange.startTime, globalTimeRange.endTime), self.inputs["pipelines"]))

    def mapSinglePipeline(self, pipelineConfig, globalStartTime, globalEndTime):
        inputCalcConfig = InputsCalcConfig(pipelineConfig["calc_grouped_stages"], pipelineConfig.get(
            "start_time", globalStartTime), pipelineConfig.get("end_time", globalEndTime))
        return Pipeline(pipelineConfig["name"], inputCalcConfig)

    def outputCustomizeClazz(self):
        absolute_class_names = self.inputs.get("output_class_name", "crawling_gocd.outputs.OutputCsv").split(";")
        output_class_names = []

        for class_name in absolute_class_names:
            partitions = class_name.strip().rpartition(".")
            output_class = getattr(sys.modules[partitions[0]], partitions[2])
            output_class_names.append(output_class)

        return output_class_names

    def getMetrics(self):
        metrics = self.inputs["metrics"]
        four_key_metrics = metrics.get("four_key_metrics", [])
        return list(map(lambda m: getattr(sys.modules["crawling_gocd.four_key_metrics"], m), four_key_metrics))
    
    def getGlobalTimeRange(self):
        return self.inputTimeParser.parse(self.inputs.get("global", {}))


class InputTimeParser:
    def parse(self, globalDict):
        type = globalDict.get("time_type", None)
        if type == "cycle":
            return self.getCycleTimeRange(globalDict)
        else:
            return self.getFixTimeRange(globalDict)

    def getFixTimeRange(self, globalDict):
        startTime = globalDict.get("start_time", datetime(1970, 1, 1))
        endTime = globalDict.get("end_time", datetime.now())
        return GlobalTimeRange(startTime, endTime)

    def getCycleTimeRange(self, globalDict):
        weekNum = globalDict.get("cycle_weeks")
        now = datetime.now().replace(hour=0, minute=0, second=0, microsecond=0)

        endTime = now - timedelta(days = now.weekday())
        startTime = endTime - timedelta(days = 7 * weekNum)
        return GlobalTimeRange(startTime, endTime - timedelta(seconds = 1))





import unittest
import json
import datetime
import tests.unit.test_fixture as fixture
from crawling_gocd.four_key_metrics import DeploymentFrequency, ChangeFailPercentage, MeanTimeToRestore
from crawling_gocd.gocd_domain import Pipeline
from crawling_gocd.crawler import CrawlingDataMapper
from crawling_gocd.calculate_domain import InputsCalcConfig


class DeploymentFrequencyTest(unittest.TestCase):
    def setUp(self):
        self.pipeline = fixture.generatePipeline()

    def test_should_calculate_deployment_frequency_correctly(self):
        handler = DeploymentFrequency()
        results = handler.calculate([self.pipeline], [])
        self.assertEqual("".join(str(x) for x in results),
                         "{ pipelineName: go_service, metricsName: DeploymentFrequency, groupName: qa, value: 6 }")


class ChangeFailPercentageTest(unittest.TestCase):
    def setUp(self):
        self.pipeline = fixture.generatePipeline()

    def test_should_calculate_change_fail_percentage_correctly(self):
        handler = ChangeFailPercentage()
        results = handler.calculate([self.pipeline], [])
        self.assertEqual("".join(str(x) for x in results),
                         "{ pipelineName: go_service, metricsName: ChangeFailPercentage, groupName: qa, value: 16.7% }")

class MeanTimeToRestoreTest(unittest.TestCase):
    def setUp(self):
        self.pipeline = fixture.generatePipeline()

    def test_should_calculate_mean_time_to_restore_correctly(self):
        handler = MeanTimeToRestore()
        results = handler.calculate([self.pipeline], [])
        self.assertEqual("".join(str(x) for x in results),
                         "{ pipelineName: go_service, metricsName: MeanTimeToRestore, groupName: qa, value: 69(mins) }")



from crawling_gocd.calculate_domain import CalculateStrategyHandler, Result

class CalculateStrategyHandlerBase(CalculateStrategyHandler):
    def calculate(self, pipelines, results):
        for pipeline in pipelines:
            self.calculateSingle(pipeline, results)
        return results

    def calculateSingle(self, pipeline, results):
        for groupedStage in pipeline.calcConfig.groupedStages.items():
            value = self.valueOfSingleGroupedStage(pipeline.histories, groupedStage[1])
            results.append(Result(pipeline.name, self.getMetricName(), groupedStage[0], value))
    
    def getMetricName(self):
        return ""

    def valueOfSingleGroupedStage(self, pipelineHistories, stageNames):
        return 0


class DeploymentFrequency(CalculateStrategyHandlerBase):
    def getMetricName(self):
        return "DeploymentFrequency"

    def valueOfSingleGroupedStage(self, pipelineHistories, stageNames):
        return len(list(filter(lambda history: history.hasStatusInStages(stageNames), pipelineHistories)))

class ChangeFailPercentage(CalculateStrategyHandlerBase):
    def getMetricName(self):
        return "ChangeFailPercentage"

    def valueOfSingleGroupedStage(self, pipelineHistories, stageNames):
        runCount = len(list(filter(lambda history: history.hasStatusInStages(stageNames), pipelineHistories)))
        failedCount = len(list(filter(lambda history: history.hasFailedInStages(stageNames), pipelineHistories)))

        if runCount == 0:
            return "N/A"

        return "{:.1%}".format(failedCount / runCount)

class ChangeFailPercentage_ignoredContinuousFailed(CalculateStrategyHandlerBase):
    def getMetricName(self):
        return "ChangeFailPercentage_2"

    def valueOfSingleGroupedStage(self, pipelineHistories, stageNames):
        runCount = len(list(filter(lambda history: history.hasStatusInStages(stageNames), pipelineHistories)))
        
        pipelineHistories.sort(key=lambda history: history.label)
        failedCount, lastIsFailed = 0, False
        for history in pipelineHistories:
            if history.hasFailedInStages(stageNames) and lastIsFailed == False:
                failedCount += 1
                lastIsFailed = True
            
            if history.allPassedInStages(stageNames) and lastIsFailed == True:
                lastIsFailed = False

        if runCount == 0:
            return "N/A"

        return "{:.1%}".format(failedCount / runCount)

class MeanTimeToRestore(CalculateStrategyHandlerBase):
    def getMetricName(self):
        return "MeanTimeToRestore"

    def valueOfSingleGroupedStage(self, pipelineHistories, stageNames):
        restoreTotalTime, failedCount, latestFailedScheduled = 0, 0, 0

        pipelineHistories.sort(key=lambda history: history.scheduledTimestamp)
        for history in pipelineHistories:
            if history.hasFailedInStages(stageNames) and latestFailedScheduled == 0:
                failedCount += 1
                latestFailedScheduled = history.scheduledTimestamp

            if history.allPassedInStages(stageNames) and latestFailedScheduled != 0:
                restoreTotalTime += history.scheduledTimestamp - latestFailedScheduled
                latestFailedScheduled = 0
        
        if latestFailedScheduled != 0 and failedCount > 0:
            failedCount -= 1

        if failedCount == 0:
            return "N/A"

        return "%s(mins)" % round(restoreTotalTime / failedCount / 1000 / 60)


    



import time
import sys
import itertools
import logging
from operator import attrgetter
from datetime import datetime
from requests_html import HTMLSession
from crawling_gocd.gocd_domain import PipelineHistory, StageHistory


class Crawler:
    def __init__(self, organization):
        self.path = "./data"
        self.organization = organization
        self.session = HTMLSession()
        self.session.headers.update(
            {"Authorization": self.organization.getBasicAuth()})

    def getResource(self, url):
        return self.session.get(url=url).json()

    def generatePipelineHistoryUrl(self, pipelineName, start=0):
        url = "https://{}/go/pipelineHistory.json?pipelineName={}".format(
            self.organization.site, pipelineName)
        if start != 0:
            url = url + "&start={}".format(start)
        return url

    def getPipelineHistories(self, pipelineName, startTime, endTime):
        if startTime > endTime:
            return []
        offset, data = 0, []
        while True:
            url = self.generatePipelineHistoryUrl(pipelineName, offset)
            logging.debug("get url {}".format(url))
            try:
                ret = self.getResource(url)
                pipelineHistoriesList = list(
                    map(lambda x: x["history"], ret["groups"]))
                pipelineHistories = list(itertools.chain(*pipelineHistoriesList))

                if len(pipelineHistories) == 0:
                    break

                data = data + self.filterPipelinesPerPage(
                        pipelineHistories, startTime, endTime)

                if self.canStop(pipelineHistories, startTime):
                    break

                offset = ret["start"] + ret["perPage"]
            except:
                logging.error("failed {} \n".format(url), sys.exc_info())
                break
        return data

    def filterPipelinesPerPage(self, pipelines, startTime, endTime):
        return list(filter(lambda x: datetime.timestamp(startTime) <= (int(x["scheduled_timestamp"]) / 1000) <= datetime.timestamp(endTime), pipelines))

    def canStop(self, pipelines, startTime):
        scheduledTimestatmps = list(
            map(lambda x: int(x["scheduled_timestamp"]) / 1000, pipelines))
        return (min(scheduledTimestatmps)) < datetime.timestamp(startTime)

class CrawlingDataMapper:
    def mapPipelineHistory(self, pipelineHistories):
        return list(map(lambda history: PipelineHistory(
            history["counterOrLabel"], history["scheduled_timestamp"], self.mapPipelineHistoryStage(history["stages"])), pipelineHistories))

    def mapPipelineHistoryStage(self, stageArray):
        return list(map(lambda stage: StageHistory(stage["stageId"], stage["stageName"], stage["stageStatus"]), stageArray))



import unittest
from freezegun import freeze_time
from crawling_gocd.inputs_parser import InputsParser, InputTimeParser
from crawling_gocd.outputs import Output, OutputCsv


class InputsParserTest(unittest.TestCase):
    def setUp(self):
        self.parser = InputsParser("tests/unit/resources/crawling-gocd.yaml")

    def test_should_generate_inputs_object_correctly_by_fix_time_range(self):
        result = self.parser.parsePipelineConfig()
        self.assertEqual("".join(str(x) for x in result),
                         "{ name: accounting-plus-master, calcConfig: { groupedStages: {'ci': ['code-scan', 'test-integration', 'build'], 'qa': ['flyway-qa', 'deploy-qa']}, startTime: 2019-07-01 00:00:00, endTime: 2019-08-12 23:59:59 } }")

    @freeze_time("2019-10-25 08:00:00")
    def test_should_generate_inputs_object_correctly_by_cycle_time_range(self):
        self.parser.inputs["global"].update(
            {
               "time_type": "cycle",
               "cycle_weeks": 2
            }
        )
        result = self.parser.parsePipelineConfig()
        self.assertEqual("".join(str(x) for x in result),
                         "{ name: accounting-plus-master, calcConfig: { groupedStages: {'ci': ['code-scan', 'test-integration', 'build'], 'qa': ['flyway-qa', 'deploy-qa']}, startTime: 2019-10-07 00:00:00, endTime: 2019-10-20 23:59:59 } }")

    def test_should_return_customize_type_class(self):
        self.parser.inputs.update(
            {"output_class_name": "crawling_gocd.outputs.Output"})
        clazz = self.parser.outputCustomizeClazz()
        self.assertTrue(isinstance(clazz[0](),  Output))

    def test_should_return_multiple_customize_type_class(self):
        self.parser.inputs.update(
            {"output_class_name": "crawling_gocd.outputs.Output; crawling_gocd.outputs.OutputCsv"}
        )
        classes = self.parser.outputCustomizeClazz()
        self.assertTrue(isinstance(classes[0](),  Output))
        self.assertTrue(isinstance(classes[1](),  OutputCsv))

    def test_should_return_default_type_class(self):
        clazz = self.parser.outputCustomizeClazz()
        self.assertTrue(isinstance(clazz[0](), OutputCsv))

    def test_should_return_metrics_class(self):
        metricsClazz = self.parser.getMetrics()
        self.assertEqual(str(
            metricsClazz), "[<class 'crawling_gocd.four_key_metrics.DeploymentFrequency'>, <class 'crawling_gocd.four_key_metrics.ChangeFailPercentage'>]")


class InputTimeParserTest(unittest.TestCase):

    @freeze_time("2019-10-07 19:00:00")
    def test_should_return_correctly_cycle_start_time_end_time(self):
        globalDict = {
            "time_type": "cycle",
            "cycle_weeks": 2
        }
        parser = InputTimeParser()
        globalTimeRange = parser.parse(globalDict)
        self.assertEqual(str(globalTimeRange.startTime), "2019-09-23 00:00:00")
        self.assertEqual(str(globalTimeRange.endTime), "2019-10-06 23:59:59")

    def test_should_return_correctly_fix_start_time_end_time(self):
        globalDict = {
            "time_type": "fix",
            "start_time": "2019-09-30 01:00:00", 
            "end_time": "2019-10-13 23:59:00"
        }
        parser = InputTimeParser()
        globalTimeRange = parser.parse(globalDict)
        self.assertEqual(str(globalTimeRange.startTime), "2019-09-30 01:00:00")
        self.assertEqual(str(globalTimeRange.endTime), "2019-10-13 23:59:00")

    



import os

from crawling_gocd.calculator import Calculator
from crawling_gocd.crawler import Crawler, CrawlingDataMapper
from crawling_gocd.gocd_domain import Organization
from crawling_gocd.inputs_parser import InputsParser


class Portal:
    def __init__(self):
        self.inputsParser = InputsParser("crawling-gocd.yaml")
        self.calculator = self.assembleCalculator()
        self.output = self.newOutputInstance()
        self.crawler = self.newCrawler()
        self.globalTimeRange = self.getGlobalTimeRange()

    def serve(self):
        input_pipelines = self.inputsParser.parsePipelineConfig()
        pipeline_with_full_data = list(map(lambda pipeline: self.crawlingSinglePipeline(
            pipeline, self.crawler), input_pipelines))

        results = self.calculator.work(pipeline_with_full_data, [])

        for o in self.output:
            o.output(results, self.globalTimeRange)

    def crawlingSinglePipeline(self, pipeline, crawler):
        mapper = CrawlingDataMapper()
        histories = crawler.getPipelineHistories(
            pipeline.name, pipeline.calcConfig.startTime, pipeline.calcConfig.endTime)
        pipeline.histories = mapper.mapPipelineHistory(histories)
        return pipeline

    def newCrawler(self):
        orgnization = Organization(
            os.environ["GOCD_SITE"], os.environ["GOCD_USER"], os.environ["GOCD_PASSWORD"])
        return Crawler(orgnization)

    def assembleCalculator(self):
        inputMetricClasses = self.inputsParser.getMetrics()
        handlers = list(map(lambda clazz: clazz(), inputMetricClasses))
        return Calculator(handlers)

    def newOutputInstance(self):
        input_output_classes = self.inputsParser.outputCustomizeClazz()
        results = []
        for o in input_output_classes:
            results.append(o())
        return results

    def getGlobalTimeRange(self):
        return self.inputsParser.getGlobalTimeRange()



import unittest
import os
from crawling_gocd.portal import Portal
from crawling_gocd.outputs import OutputCsv

class PortalTest(unittest.TestCase):
    def setUp(self):
        os.environ["GOCD_SITE"] = "test.com"
        os.environ["GOCD_USER"] = "test_user"
        os.environ["GOCD_PASSWORD"] = "123456"

    def test_new_crawler_correctly(self):
        self.assertIsNotNone(Portal().crawler)

    def test_new_crawler_failed(self):
        os.environ.pop("GOCD_SITE", None)
        os.environ.pop("GOCD_USER", None)
        os.environ.pop("GOCD_PASSWORD", None)
        self.assertRaises(KeyError, Portal)

    def test_new_calculator_correctly(self):
        calculator = Portal().calculator
        self.assertEqual(len(calculator.strategyHandlers), 4)

    def test_new_output_instance_correctly(self):
        self.assertTrue(type(Portal().output[0]) == OutputCsv)

    def test_get_global_time_range_correctly(self):
        self.assertIsNotNone(Portal().getGlobalTimeRange())

    


import unittest
import json
import tests.unit.test_fixture as fixture
from crawling_gocd.crawler import CrawlingDataMapper
from crawling_gocd.gocd_domain import PipelineHistory, StageHistory

class CrawlingDataMapperTest(unittest.TestCase):
    def setUp(self):
        filePage1 = "tests/unit/resources/pipeline_history_pg_1.json"
        self.pipelineHistories = fixture.getPipelineHistories(filePage1)
        self.mapper = CrawlingDataMapper()

    def test_should_map_domain_object_correctly(self):
        result = self.mapper.mapPipelineHistory(self.pipelineHistories)
        givenData = PipelineHistory("226", "1567335377736", [
            StageHistory("217936", "code-scan", "Passed"),
            StageHistory("217937", "test-integration", "Passed"),
            StageHistory("217939", "build", "Passed"),
            StageHistory("217982", "flyway-qa", "Passed"),
            StageHistory("217983", "deploy-qa", "Passed")
        ])
        self.assertEqual(str(result[0]), str(givenData))

if __name__ == '__main__':
    unittest.main()


import unittest
import csv
from datetime import datetime
from crawling_gocd.outputs import OutputCsv
from crawling_gocd.calculate_domain import GlobalTimeRange
import tests.unit.test_fixture as fixture

class OutputCsvTest(unittest.TestCase):
    def setUp(self):
        self.outputCsv = OutputCsv()
        self.globalTimeRange = GlobalTimeRange(datetime(2019, 10, 15, 0), datetime(2019, 10, 17, 23, 59, 59))
        self.fileName = "crawling_output_20191015000000_20191017235959.csv"

    def test_output(self):
        self.outputCsv.output(fixture.getResults(), self.globalTimeRange)
        with open(self.fileName, newline="") as csvFile:
            reader = csv.reader(csvFile)
            self.assertEqual(str(list(reader)), "[['pipelineName', 'groupName', 'ChangeFailPercentage', 'DeploymentFrequency', 'MeanTimeToRestore'], ['account-management-normal-master', 'ci', '5.5%', '145', '56(mins)'], ['account-management-normal-master', 'qa', '4.9%', '61', '53(mins)']]")


import base64
from crawling_gocd.calculate_domain import InputsCalcConfig


class Organization:
    def __init__(self, site, user, token):
        self.site = site
        self.user = user
        self.token = token

    def getBasicAuth(self):
        return "Basic {}".format(base64.b64encode((self.user + ":" + self.token).encode('utf-8')).decode())


class Pipeline:
    def __init__(self, name, calcConfig: InputsCalcConfig):
        self.name = name
        self.calcConfig = calcConfig

    def __str__(self):
        return "{ name: %s, calcConfig: %s }" % (self.name, str(self.calcConfig))
    
    def setHistories(self, histories):
        self.histories = histories
    
class PipelineHistory:
    def __init__(self, label, scheduledTimestamp, stages):
        self.label = label
        self.scheduledTimestamp = scheduledTimestamp
        self.stages = stages

    def __str__(self):
        return "{ label: %s, scheduledTimestamp: %s, stages: %s }" % (self.label, str(self.scheduledTimestamp), str(", ".join(str(stage) for stage in self.stages)))

    def hasStatusInStages(self, stageNames):
        return len(list(filter(lambda stage: stage.name in stageNames and stage.status not in ("Unknown", "Cancelled"), self.stages))) > 0
    
    def hasFailedInStages(self, stageNames):
        return len(list(filter(lambda stage: stage.name in stageNames and stage.status == "Failed", self.stages))) > 0

    def allPassedInStages(self, stageNames):
        return len(list(filter(lambda stage: stage.name in stageNames and stage.status == "Passed", self.stages))) == len(stageNames)


class StageHistory:
    def __init__(self, oid, name, status):
        self.id = oid
        self.name = name
        self.status = status

    def __str__(self):
        return "{ id: %s, name: %s, status: %s}" % (self.id, self.name, self.status)



from datetime import datetime

class InputsCalcConfig:
    def __init__(self, groupedStages: dict, startTime: datetime, endTime: datetime):
        self.groupedStages = groupedStages
        self.startTime = startTime
        self.endTime = endTime

    def __str__(self):
        return "{ groupedStages: %s, startTime: %s, endTime: %s }" % (str(self.groupedStages), str(self.startTime), str(self.endTime))

class CalculateStrategyHandler:
    def calculate(self, pipelines, results):
        return []

class GlobalTimeRange:
    def __init__(self, startTime, endTime):
        self.startTime = startTime
        self.endTime = endTime

class Result:
    def __init__(self, pipelineName, metricsName, groupName, value):
        self.pipelineName = pipelineName
        self.metricsName = metricsName
        self.groupName = groupName
        self.value = value

    def __str__(self):
        return "{ pipelineName: %s, metricsName: %s, groupName: %s, value: %s }" % (self.pipelineName, self.metricsName, self.groupName, self.value) 


import pathlib
from setuptools import setup, find_packages

HERE = pathlib.Path(__file__).parent
print("here=>", HERE)
README = (HERE / "readme.pypi.md").read_text()

setup(
    name="crawling-gocd",
    version="1.2.4",
    description="crawling the gocd build histories of pipelines and calculate the metrics",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/play-code-tools/crawling-gocd",
    author="play-code-tools",
    author_email="ellendan000@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=find_packages(exclude=("tests",)),
    install_requires=["requests-html", "pyyaml", "pyfunctional"],
    entry_points={
        "console_scripts": [
            "realpython=crawling_gocd.__main__:main",
        ]
    },
)


import csv
import itertools
import logging
from crawling_gocd.calculate_domain import Result


class Output:
    def output(self, results, globalTimeRange):
        pass

class OutputCsv(Output):
    def __init__(self):
        self.fieldNames = ["pipelineName", "groupName"]
        self.fileNameTemplate = "crawling_output_{}_{}.csv"

    def output(self, results, globalTimeRange):
        metricNames = set(list(map(lambda result: result.metricsName, results)))
        self.fieldNames += sorted(metricNames)
        formatOutputs = self.convertToFormatOutputs(results)

        fileName = self.generateCvsFileName(globalTimeRange)
        with open(fileName, mode="w") as csvFile:
            writer = csv.DictWriter(csvFile, fieldnames=self.fieldNames, delimiter=',',
                                    quotechar='"', quoting=csv.QUOTE_MINIMAL)
            writer.writeheader()
            writer.writerows(formatOutputs)
        logging.info("The metrics results see the {} file".format(fileName))

    def convertToFormatOutputs(self, results):
        keyFunction = lambda r: (r.pipelineName, r.groupName)
        results = sorted(results, key=keyFunction)
        
        formatOutputs = []
        for k, group in itertools.groupby(results, keyFunction):
            output = {"pipelineName": k[0], "groupName": k[1]}
            
            for t in sorted(group, key=lambda t: t.metricsName):
                output.update({t.metricsName: t.value})
            formatOutputs.append(output)
        return formatOutputs

    def generateCvsFileName(self, globalTimeRange):
        timeFormater = "%Y%m%d%H%M%S"
        start = globalTimeRange.startTime.strftime(timeFormater)
        end = globalTimeRange.endTime.strftime(timeFormater)
    
        return self.fileNameTemplate.format(start, end)


class OutputTest(Output):
    def __init__(self):
        print("hello world")
    def output(self, results, global_time_range):
        print("it's in the output test")


from crawling_gocd.gocd_domain import Pipeline

class Calculator:
    def __init__(self, strategyHandlers):
        self.strategyHandlers = strategyHandlers

    def work(self, pipelines: Pipeline, results = []):
        for handler in self.strategyHandlers:
            handler.calculate(pipelines, results)
        return results
